#
# ZWiFL
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
from Globals import HTML

class WiFLBlockTag:
    """
    Wifl Widget Tag
    """

    name='widget'
#    blockContinuations=()

    def __init__(self,args):
        self.args=parse_params(args, ctag='', lib='', help='')
        self.tag=self.getarg_('ctag', '')
        self.lib=self.getarg_('lib', '')
        self.help=self.getarg_('help', '')

    def getarg_(self, name, defval):
        if self.args.has_key(name):
            return self.args[name]
        else:
            return defval

    def FindAnyWiflLibrary(self,root):
        while root is not None:
            if hasattr(root,'objectValues'):
                for o in root.objectValues():
                    try:
                        if (hasattr(o,'meta_type') and
                            o.meta_type=='WiFL Component Library'):
                            return o
                    except:
                        pass
            if hasattr(root,'aq_parent'):
                root=root.aq_parent
            else:
                root=None
        return None

    def FindWiflLibraryByName(self,root):
        while root is not None:
            if hasattr(root,'objectValues'):
                for o in root.objectValues():
                    try:
                        if (o.id == self.lib and
                            (hasattr(o,'meta_type') and
                            o.meta_type=='WiFL Component Library')):
                            return o
                    except:
                        pass
            if hasattr(root,'aq_parent'):
                root=root.aq_parent
            else:
                root=None
        return None
        
    def render(self,md):
        """ Do the Translation """
        us=md.this

        print "Processing: "+self.tag
        if self.lib:
            print "Finding: "+self.lib
            lib=self.FindWiflLibraryByName(us)
        else:
            print "Finding any"
            lib=self.FindAnyWiflLibrary(us)

        if not lib:
            return ' Failed: tag='+self.tag+' Lib='+self.lib

        thisTag=getattr(lib,self.tag)
        if self.help:
            return thisTag.getDescription()
        return thisTag.__call__()
    
    __call__=render


String.commands['widget'] = WiFLBlockTag
           
