#
# ZWiFL
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#

from Globals import Persistent, HTMLFile, HTML, MessageDialog
import Globals
from Products.ZSQLMethods.SQL import SQL
import Acquisition
from OFS.SimpleItem import SimpleItem
from OFS.DTMLMethod import DTMLMethod
from DocumentTemplate.DT_Var import sql_quote
from urllib import quote
import time
from time import gmtime
import DateTime

#
# Essentially:-
#
# <dtml-var WiFLComponentId>
#

def manage_addWiFLComponent(self, id, template, dtmlArgs='',
                            template_header='',
                            template_footer='',title='',
                            sqlTemplate=None, sqlArgs='',
                            connection=None, description='',
                            componentConnection=None,
                            submit=None,REQUEST=None):
    ''' Add one of these Puppies Somewhere '''
    
    if componentConnection:
        # If we want to be stored in a database...
        realConnection=getattr(self,componentConnection)

        realConnection().query(InsertComponent%(sql_quote(id),
                                                sql_quote(template_header),
                                                sql_quote(template),
                                                sql_quote(template_footer),
                                                sql_quote(dtmlArgs),
                                                sql_quote(title),
                                                sql_quote(sqlTemplate),
                                                sql_quote(sqlArgs),
                                                sql_quote(connection),
                                                sql_quote(description),
                                                sql_quote(componentConnection),
                                                0,0
                                                
                                                ))
    else:
        # Otherwise store us in the ZODB
        o=WiFLComponent(id,template,dtmlArgs,template_header,
                        template_footer, title,sqlTemplate, sqlArgs,
                        connection, description)
        id=self._setObject(id,o)

    if REQUEST:
        try:
            u=self.DestinationURL()
        except:
            u=REQUEST['URL1']
        if submit==" Add and Edit " and not componentConnection:
            u='%s/%s'%(u,quote(id))
            REQUEST.RESPONSE.redirect(u+'/manage_edit')
        else:
            REQUEST.RESPONSE.redirect(u+'/manage_main')
        
    return ''

InsertComponent='''
INSERT INTO WiFLComponents
VALUES ('%s','%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s',
        '%s', '%s', %d, %d)
'''

UpdateComponent='''
UPDATE WiFLComponents
SET template_header='%s', template='%s', template_footer='%s',
    dtmlArgs='%s', title='%s', sqlTemplate='%s', sqlArgs='%s',
    sqlConnection='%s', description='%s', componentConnection='%s',
    cacheRendered=%d, ttl=%d
WHERE id='%s'
'''
manage_addWiFLComponentForm=HTMLFile('manage_addWiFLComponentForm',globals())

class WiFLComponentV(SimpleItem,
                     Acquisition.Implicit):

    ''' A Component to be rendered '''

    icon="misc_/WiFLZ/WiflWidget.gif"
    meta_type='Virtual WiFL Component'

    manage_edit=HTMLFile('manage_editWiFLComponentForm',globals())
    manage_edit_advanced=HTMLFile('manage_editWiFLComponentAdvancedForm',globals())
    manage_test=HTMLFile('manage_testWidget',globals())
    index_html=None
    manage_options=(
        {'label':'Edit', 'action':'manage_edit'},
        {'label':'Advanced', 'action':'manage_edit_advanced'},        
        {'label':'Test', 'action':'manage_test'},
        {'label':'Security', 'action':'manage_access'},
        {'label':'Undo', 'action':'manage_UndoForm'},
        {'label':'Find', 'action':'manage_findFrame', 'target':'manage_main'},
    )
    
    def __init__(self, id, template, dtmlArgs='', template_header='',
                 template_footer='', title='',sqlTemplate=None,
                 sqlArgs='', connection=None, description='', componentConnection='',cacheRendered=0, ttl=0):

        self.id=id
        self.title=title
        self.template_header=template_header
        self.template=template
        self.template_footer=template_footer
        self.dtmlArgs=dtmlArgs
        self.sqlTemplate=sqlTemplate
        self.sqlArgs=sqlArgs
        self.connection=connection
        self.description=description
        self.componentConnection=componentConnection
        self.cacheRendered=cacheRendered
        self.ttl=ttl
        self.DT_template=DTMLMethod()
        self.DT_template.manage_edit(data=template, title=title)
        self.DT_template_header=DTMLMethod()
        self.DT_template_header.manage_edit(data=template_header, title=title+' Header')
        self.DT_template_footer=DTMLMethod()
        self.DT_template_footer.manage_edit(data=template_footer, title=title+' Footer')
        
        if sqlTemplate and connection:
            self.addQuery(sqlTemplate,sqlArgs,connection)

    def addQuery(self, sqlTemplate, sqlArgs, sqlConnection):
        self.componentQuery=SQL('sql'+self.id,
                                'sql'+self.title,
                                sqlConnection,
                                sqlArgs,
                                sqlTemplate)

    def insertOrUpdateSQL(self, template, dtmlArgs='',
                          template_header='', template_footer='',
                          title='', sqlTemplate='',
                          sqlArgs='', connection='',
                          componentConnection='',
                          description='', cacheRendered=0,ttl=0):
        try:
            # Try a blind insert in case the user is converting
            # to a dual-mode widget
            realConnection=getattr(self,componentConnection)
            realConnection().query(
                InsertComponent%(sql_quote(self.id),
                                 sql_quote(template_header),
                                 sql_quote(template),
                                 sql_quote(template_footer),
                                 sql_quote(dtmlArgs),
                                 sql_quote(title),
                                 sql_quote(sqlTemplate),
                                 sql_quote(sqlArgs),
                                 sql_quote(connection),
                                 sql_quote(description),
                                 sql_quote(componentConnection),
                                 cacheRendered,
                                 ttl
                                 ))
        except:
            # If the above failed, try an update
            # 
            realConnection().query(
                UpdateComponent%(sql_quote(template_header),
                                 sql_quote(template),
                                 sql_quote(template_footer),
                                 sql_quote(dtmlArgs),
                                 sql_quote(title),
                                 sql_quote(sqlTemplate),
                                 sql_quote(sqlArgs),
                                 sql_quote(connection),
                                 sql_quote(description),
                                 sql_quote(componentConnection),
                                 cacheRendered,
                                 ttl,
                                 sql_quote(self.id)
                                 ))
        
    def manage_editWiFLComponent(self, template, dtmlArgs='',
                                 template_header='', template_footer='',
                                 title='', sqlTemplate=None,
                                 sqlArgs='', connection=None,
                                 componentConnection=None,
                                 description='', REQUEST=None):
        ''' Update a Widget '''
        self.template_header=template_header
        self.template=template
        self.template_footer=template_footer
        self.sqlTemplate=sqlTemplate
        self.sqlArgs=sqlArgs
        self.dtmlArgs=dtmlArgs
        self.connection=connection
        self.componentConnection=componentConnection
        self.description=description
        self.DT_template.manage_edit(data=template, title=title)
        self.DT_template_header.manage_edit(data=template_header,
                                            title=title+' Header')
        self.DT_template_footer.manage_edit(data=template_footer,
                                            title=title+' Footer')

        if sqlTemplate and connection:
            self.addQuery(sqlTemplate,sqlArgs,connection)

        if componentConnection:
            # If we want to be stored in a database...
            self.insertOrUpdateSQL(template, dtmlArgs, template_header,
                                   template_footer, title, sqlTemplate,
                                   sqlArgs, connection,
                                   componentConnection, description,
                                   self.cacheRendered)
        if REQUEST:
            return self.manage_edit(self,REQUEST)

    def manage_editWiFLComponentAdvanced(self, cacheWidget=0, ttl=0,
                                         componentConnection='',
                                         REQUEST={}):
        ''' Update Advanced Properties '''

        if cacheWidget:
            self.ttl=ttl
            self.cacheRendered=cacheWidget
            self.componentConnection=componentConnection
            self.insertOrUpdateSQL(self.template, self.dtmlArgs,
                                   self.template_header,
                                   self.template_footer,
                                   self.title, self.sqlTemplate,
                                   self.sqlArgs, self.connection,
                                   self.componentConnection,
                                   self.description,
                                   self.cacheRendered,
                                   self.ttl)
        
        if REQUEST:
            return self.manage_edit_advanced(self,REQUEST)

    def fetchCachedResult(self):
        ''' Get the cached result of this widget '''

        # MySQL has a NOW() function we can use
        # MS_SQL had a GETDATE() function we can use
        #
        # But since I want to try to be DB independent we'll
        # Grab the date and format it
        cacheQuery="""
        SELECT * from ComponentCache
        WHERE id='%s'
        AND expires > '%s'
        """

        now=time.strftime("%Y-%m-%d %H:%M:%S",gmtime(time.time()))
        
        realConnection=getattr(self,self.componentConnection)
        try:
            results=realConnection().query(
                cacheQuery%(sql_quote(self.id),
                            sql_quote(now)))
        except:
            return None

        if not results:
            return None
        
        if len(results) < 2: # One for the results...
            return None

        for tuple in results[1]:
            r=tuple[1]
            return r

    def updateCachedResult(self, r):
        ''' Update the cached result '''

        cacheUpdate="""
        UPDATE ComponentCache
        SET renderedWidget='%s',
            expires='%s'
        WHERE id='%s'
        """

        cacheInsert="""
        INSERT INTO ComponentCache
        VALUES ('%s', '%s', '%s')
        """
        
        now=time.strftime("%Y-%m-%d %H:%M:%S",gmtime(time.time()+self.ttl))
        
        realConnection=getattr(self,self.componentConnection)

        try:
            realConnection().query(cacheInsert%(self.id, r, now))
        except:
            realConnection().query(cacheUpdate%(r, now, self.id))
            
        
    def __call__(self,client=None, REQUEST={}, RESPONSE=None, **kw):
        ''' wheeeeee '''

        if kw['help']:
            return self.description
        
        r=''
        if self.cacheRendered:
            r=self.fetchCachedResult()

        if r:
            return r
        
        if REQUEST is None:
            if kw:
                REQUEST=kw
            elif hasattr(self,'REQUEST'):
                REQUEST=self.REQUEST

        r=self.DT_template_header.__call__(REQUEST=REQUEST,
                                           RESPONSE=RESPONSE,kw=kw)
        if self.componentQuery:
            objects=self.componentQuery(REQUEST)
            for object in objects:
                for key in objects.names():
                    kw[key]=sqlattr(object,key)
                r=r+(self.DT_template.__call__(client=None,REQUEST=kw,
                                               RESPONSE=None))
        else:
            r=r+self.DT_template.__call__(REQUEST=REQUEST,
                                          RESPONSE=RESPONSE,kw=kw)

        r=r+self.DT_template_footer.__call__(REQUEST=REQUEST,
                                             RESPONSE=RESPONSE,kw=kw)

        if self.cacheRendered:
            self.updateCachedResult(r)
        return r

    def testWidget(self,REQUEST={}):
        ''' Test the Widget '''
        if REQUEST is None:
            if kw:
                REQUEST=kw
            elif hasattr(self,'REQUEST'):
                REQUEST=self.REQUEST
        r='<HTML><HEAD></HEAD><BODY BGCOLOR="WHITE">'
        r=r+self.__call__(REQUEST=REQUEST)
        r=r+'</BODY></HTML>'
        return r

    widget_test=testWidget
    
    def render(self,REQUEST={}, RESPONSE=None, **kw):
        return self.DT.__call__(REQUEST=REQUEST,RESPONSE=RESPONSE,kw=kw)

    def getTemplateHeader(self,REQUEST=None):
        ''' Return the HTML Header Code '''
        return self.template_header

    def getTemplate(self, REQUEST=None):
        ''' Return the HTML Template Code '''
        return self.template

    def getTemplateFooter(self,REQUEST=None):
        ''' Return the HTML Footer Code '''
        return self.template_footer
    
    def getSqlTemplate(self, REQUEST=None):
        ''' Return the SQL Template Code '''
        return self.SqlTemplate

    def getSqlParms(self, REQUEST=None):
        ''' Return the SQL Parameters '''
        return self.sqlArgs
    def getDtmlParms(self,REQUEST=None):
        ''' Return the DTML Paramaters '''
        return self.dtmlArgs

    def getDescription(self,REQUEST=None):
        ''' Get the component Description '''
        return self.description

class WiFLComponent(WiFLComponentV,Persistent):
    ''' Persistent WiFLComponents '''
    meta_type='WiFL Component'
    
from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name
        

