#
# ZWiFL
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
#

import Globals
from Globals import Persistent, HTMLFile

from OFS.Folder import Folder
from Products.ZSQLMethods.SQL import SQL
from WiflComponent import WiFLComponentV
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse



def manage_addWiFLLibrary(self, id, connection,title='', REQUEST={}):
    """ Add a New Wifl Component Library """

    o=WiFLLibrary(id, connection, title)
    self._setObject(id,o)

    if REQUEST:
        return self.manage_main(self,REQUEST)
    return ' '

manage_addWiFLLibraryForm=HTMLFile('manage_addWiFLLibraryForm',globals())

_marker=[]

class WiFLLibrary(Folder):
    """ Front End for SQL-Based WiFL Widgets """

    meta_type='WiFL Component Library'

    manage_options=(
        {'label':'Contents', 'action':'manage_main'},
        {'label':'View', 'action':'index_html'},
        {'label':'Security', 'action':'manage_access'},
        {'label':'Undo', 'action':'manage_UndoForm'},
        {'label':'Find', 'action':'manage_findFrame', 'target':'manage_main'},
    )

    __ac_permissions__=(
        ('View management screens',('manage_tabs', 'manage_main',)),
        ('Change permissions', ('manage_access')),
        ('View',('',)),
        )


    def __init__(self, id, connection, title=''):
        """ Initialise the Library """
        self.id=id
        self.title=title
        self.connection=connection
        self.createSQLQueries()

    def createSQLQueries(self):
        """ Because it didn't have one either """
        self.sqlWiflComponentByIdQuery=SQL(
            'sqlWiflComponentByIdQuery',
            'Get One Wifl Component',
            self.connection,
            'wid:required',
            _sqlWiflComponentByIdQuery)

        self.sqlWiflComponentQuery=SQL(
            'sqlWiflComponentQuery',
            'Get All Wifl Component Ids',
            self.connection,
            '',
            _sqlWiflComponentQuery)
        self.sqlDeleteWiflComponent=SQL(
            'sqlDeleteWiflComponent',
            'Delete One Wifl Component',
            self.connection,
            'wid:required',
            _sqlDeleteWiflComponent)

    def _getWiflComponentOb(self,id,spec=None):
        """ Because it didn't have one """
        data = self.sqlWiflComponentByIdQuery(wid=id)
        for ob in data:
            wiflComponent=WiFLComponentV(
                sqlattr(ob,'id'),
                sqlattr(ob,'template'),
                sqlattr(ob,'dtmlArgs'),
                sqlattr(ob,'template_header'),
                sqlattr(ob,'template_footer'),
                sqlattr(ob,'title'),
                sqlattr(ob,'sqlTemplate'),
                sqlattr(ob,'sqlArgs'),
                sqlattr(ob,'sqlConnection'),
                sqlattr(ob,'description'),
                sqlattr(ob,'componentConnection'),
                sqlattr(ob,'cacheRendered'),
                sqlattr(ob,'ttl'))
            setattr(self,id,wiflComponent)
            return wiflComponent

    def _getOb(self, id, default=_marker):
        """ Get more of my Bohects """
        return self._getWiflComponentOb(id)
    
    def _my_getOb(self, name, default=_marker):
        """ Get one of my Objects """
        try:
            return self._getOb(name,default)
        except (AttributeError, ValueError):
            HTTPResponse().notFoundError(name)
        except:
            raise
    
    def _delObject(self, id, dp=1):
        object=self._getOb(id)
        if (hasattr(object,'meta_type') and
            getattr(object,'meta_type')=='Virtual WiFL Component'):
            self.sqlDeleteWiflComponent(wid=id)
        else:
            object.manage_beforeDelete(object, self)
            self._objects=tuple(filter(lambda i,n=id: i['id']!=n, self._objects))
            self._delOb(id)

    def tpId(self):
        """ tpId """
        return self.id

    def __get_item__(self,i):
        """ Get the item damnit """
        return self._my_getOb(i)

    def objectIds(self, spec=None):
        """ Return a list of subobject ids. """
        set = []

        data=self.sqlWiflComponentQuery()
        if not data:
            return set

        for ob in data:
            set.append(str(sqlattr(ob,'id')))
        return set

    def _ids(self,spec=None):
        """ The same as objectIds? """
        return self.objectIds(spec)
        
    def objectValues(self, spec=None):
        """ Return a list of Values """
        r=[]
        a=r.append
        g=self._my_getOb
        ourIds=self.objectIds(spec)
        if ourIds:
            for id in ourIds:
                a(g(id,spec))
        return r
    
        
from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name

_sqlWiflComponentQuery="""
SELECT id
FROM WiFLComponents
"""
_sqlWiflComponentByIdQuery="""
SELECT *
FROM WiFLComponents
WHERE id=<dtml-sqlvar wid type=nb>
"""

_sqlDeleteWiflComponent="""
DELETE
FROM WiFLComponents
WHERE id=<dtml-sqlvar wid type=nb>
"""
